/******************************************************************************
 *  FileName:       IS01BFRGB_BitBang.c
 *  Dependencies:
 *  Processor:      ATTINY13A
 *  Compiler:       WinAVR 20081205
 *
 * SOFTWARE IS PROVIDED "AS IS." DIGI-KEY EXPRESSLY DISCLAIMS ANY
 * WARRENTY OF ANY KIND, WHETHER EXPRESS OF IMPLIED, INCLUDING BUT NOT
 * LIMITED TO, THE IMPLIED WARRENTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL DIGI-KEY
 * BE LIABLE FOR A INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES, LOST PROFITS, OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
 * PROCUREMENT OF SUBSTITUTE GOOD, TECHNOLOGY OF SERVICES, AND CLAIMS
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
 * ANY CLAIMS FOR INDEMNITY OF CONTRIBUTION, OR OTHER SIMILAR COSTS.
 * DIGI-KEY ALSO DISCLAIMS ANY LIABILITY FOR PATENT OF COPYRIGHT
 * INFRINGEMENT.
 *
 * Version History
 * Version 1.0 05/01/2009 Brandon Tougas
 * Initial Public Release
 ******************************************************************************/

#include <avr/io.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include <avr/fuse.h>

#define     TRUE    1
#define     FALSE   0
#define     HIGH    1
#define     LOW     0

void Initialize_TIMER0(void);
void Initialize_PORTS(void);
void Initialize_INTERRUPTS(void);

volatile signed char c_Bit = 0;
volatile signed char c_Row = 0;
volatile signed char c_Column = 4;

volatile unsigned char uc_Din = 0;
volatile unsigned char uc_Temp = 0;
volatile unsigned char uc_Bit40 = 0;
volatile unsigned char uc_Display = 0;
volatile unsigned char uc_Switch_Up = FALSE;
volatile unsigned char uc_Switch_Down = FALSE;
volatile unsigned char uc_Endof_Lcd_Frame = FALSE;
volatile unsigned char uc_Scp_State = LOW;

extern unsigned char IS01BFRGB_bitmap_1[24][5] PROGMEM;
extern unsigned char IS01BFRGB_bitmap_2[24][5] PROGMEM;

#define IS01BFRGB_DIN   PORTB1      // Data In pin of the IS01BFRGB
#define IS01BFRGB_SCP   PORTB0      // Serial Clock Pulse pin of the IS01BFRGB
#define IS01BFRGB_LP    PORTB2      // Latch Pulse pin of the IS01BFRGB
#define IS01BFRGB_FLM   PORTB4      // First Line Marker pin of the IS01BFRGB

#define SWITCH          PORTB3
#define S_PORT          PINB

/*******************************************************************************
 *
 *      Function Name:  Fuses
 *
 *      Returns: None
 *
 *      Parameters: None
 *
 *      Purpose: To set the devices fuses within the code
 *
 ******************************************************************************/

FUSES =
{
    .low = (FUSE_SPIEN & FUSE_SUT1 & FUSE_SUT0 & FUSE_CKSEL0),

    .high = (FUSE_BODLEVEL1)
};

/******************************************************************************
 *
 *      Function Name:  Pin-Change ISR
 *
 *      Returns: None
 *
 *      Parameters: None
 *
 *      Purpose: To change the bitmap lookup when external button is pressed
 *
 ******************************************************************************/

ISR(PCINT0_vect, ISR_BLOCK)
{
    PCMSK &= ~(1 << PCINT3);      // Disable PA7 for a Pin Change Interrupt

    if((!(S_PORT & (1 << SWITCH)) == LOW))
    {
        GTCCR |= (1 << TSM);      // Stop Timer0

        if (uc_Display == 0)
        {
            uc_Display = 1;
        }
        else
        {
            uc_Display = 0;
        }

        uc_Bit40 = 0;  // Reset bit to 1 for the shift register

        c_Bit = 0;     // Reset bit to zero

        c_Column = 4;  // Reset column to four

        c_Row = 0;     // Reset row to zero

        PORTB &= !(1 << IS01BFRGB_SCP);

        uc_Scp_State = LOW;

        uc_Switch_Up = FALSE;

        uc_Switch_Down = TRUE;

        TIFR0 |= (1 << OCF0A);  // Clear Output A compare flag

        GTCCR &= ~(1 << TSM);   // Start and clear Timer0
    }

    PCMSK |= (1 << PCINT3);     // Enable PA7 for a Pin Change Interrupt

}

/******************************************************************************
 *
 *      Function Name:  Timer0 Compare A ISR
 *
 *      Returns: None
 *
 *      Parameters: None
 *
 *      Purpose: To get and set the data bits on Din and to control LP anf FLM
 *
 *****************************************************************************/

ISR(TIM0_COMPA_vect, ISR_BLOCK)   // Timer 3 compare interrupt vector
{

    if (uc_Scp_State == LOW)
    {
        uc_Scp_State = HIGH;
    }
    else
    {
        uc_Scp_State = LOW;
    }

    /* Execute when the clock is low */
    if (uc_Scp_State == LOW)
    {
        if (uc_Din == LOW)
        {
            PORTB &= ~(1 << IS01BFRGB_DIN);     // Force PORTD0 low
        }
        else
        {
            PORTB |= (1 << IS01BFRGB_DIN);      // Force PORTD0 high
        }


        if (uc_Bit40 == 38)
        {
            if (c_Row == 0)
            {
                // Force FLM pin high to indicate Row 1 of the Display
                PORTB |= (1 << IS01BFRGB_FLM); 
            }

            PORTB |= (1 << IS01BFRGB_LP);
        }

        if (uc_Bit40 >= 40)
        {
            PORTB &= ~(1 << IS01BFRGB_LP); // of the line/row of the Display

            PORTB &= ~(1 << IS01BFRGB_FLM);    // Force FLM low

            uc_Bit40 = 0;   // Reset bit to 0 for the shift register
        }

        uc_Bit40++;     // Increment variable by one
    }

    /* Execute when the clock is high, This routine will retrieve
       the bit map data from right to left and from top to bottom */
    else
    {
        if (c_Bit == 0)
        {
            if (!uc_Display)
            {
                /* Retrieve data from bit map, Invert or ones complement "~" */
                uc_Temp = 
                      ~(pgm_read_byte(&(IS01BFRGB_bitmap_2[c_Row][c_Column])));
            }
            else{
                /* Retrieve data from bit map */
                uc_Temp = 
                      (pgm_read_byte(&(IS01BFRGB_bitmap_1[c_Row][c_Column])));
            }
        }

        //shift data right,AND it with 0x01, uc_Din = 00000000 or 00000001 
        uc_Din = ((uc_Temp >> c_Bit) & 0x01); 

        c_Bit++;   // Increment to retrieve the next bit from the bit map

        if (c_Bit >= 8)
        {
            c_Bit = 0;     // Reset bit to zero after every eighth bit
            c_Column--;    // Next column of the bit map

            //Reset column to 4 after every 5th column of the bitmap
            if (c_Column < 0) 
            { 
                c_Column = 4;
                c_Row++;       // Next row of the bit map

                //Reset row to 0 after every 24th row of the bitmap
                if (c_Row >= 24) 
                {        
                    c_Row = 0;
                }
            }
        }
    }

    PORTB ^= (1 << IS01BFRGB_SCP);
}

/******************************************************************************
 *
 *      Function Name:  Initialize_INTERRUPTS
 *
 *      Returns: None
 *
 *      Parameters: None
 *
 *      Purpose:   Configure the pin change interrupt on PA7 and enable
 *                 Global interrupts.
 *
 ******************************************************************************/

void Initialize_INTERRUPTS(void)
{
    cli();      // Clear global interrupts

    // Enable Pin Change Interrupt 0, On PORTA
    GIMSK |= ((0 << INT0) | (1 << PCIE));

    // Clear Pin Change Interrupt Flag 0,   
    GIFR |= (1 << PCIF);

    PCMSK |= (1 << PCINT3);  // Enable PA7 for a Pin Change Interrupt

    sei();      // Enable global interrupts
}

/******************************************************************************
 *
 *      Function Name:  Initialize_TIMER0
 *
 *      Returns: None
 *
 *      Parameters: None
 *
 *      Purpose: Timer 0 is used as an output clock for the display. 
 *               See datasheet for details about each register.
 *               (CTC = Clear Timer on Compare, Clock divided by 8, 
 *               Set Output Compare register, Toggle OC0 on compare match, 
 *               Enable and clear Output Compare match interrupt) 
 *
 ******************************************************************************/

void Initialize_TIMER0(void)
{

    /* Normal Port Operation, OC0A disconnected, CTC Mode */
    TCCR0A |= ( 1 << WGM01 );

    /* Set clk/8 */
    TCCR0B |= ((0 << WGM02) | (0 << CS02) | (1 << CS01) | (0 << CS00));

    TCNT0 = 0x00;

    OCR0A = 0x11;   // Determines the SCP clock speed

    TIMSK0 |= ((0 << OCIE0B) | (1 << OCIE0A) | (0 << TOIE0));

}

/******************************************************************************
 *
 *      Function Name:  Initialize_PORTS
 *
 *      Returns: None
 *
 *      Parameters: None
 *
 *      Purpose:    To setup the I/Os
 *
 ******************************************************************************/

void Initialize_PORTS(void)
{
    /***************************** PORT B *************************************/

    DDRB |= ((1 << DDB0) | (1 << DDB1) | (1 << DDB2) | (0 << DDB3)
             | (1 << DDB4) | (1 << DDB5));

    PORTB |= ((0 << PORTB0) | (0 << PORTB1) | (0 << PORTB2) | (0 << PORTB3)
              | (0 << PORTB4) | (1 << PORTB5));
}

/******************************************************************************
 *
 *      Function Name:  Main
 *
 *      Returns: None
 *
 *      Parameters: None
 *
 *      Purpose: To initialize the peripherals used and wait for an interrupt
 *
 ******************************************************************************/

int main(void)
{
    Initialize_PORTS();

    Initialize_TIMER0();

    Initialize_INTERRUPTS();

    for (;;)
    {

    }

    return(0);
}
